#include "cproj.h"

/* Variables common to all subroutines in this code file
  -----------------------------------------------------*/
static double lon_center;       /* Center longitude (projection center) */
static double R;                /* Radius of the earth (sphere) */
static double false_easting;    /* x offset in meters                   */
static double false_northing;   /* y offset in meters                   */

/* Initialize the Mollweide projection
  ------------------------------------*/
winkint(r, center_long, false_east, false_north) 
double r;                       /* (I) Radius of the earth (sphere) */
double center_long;             /* (I) Center longitude */
double false_east;              /* x offset in meters                   */
double false_north;             /* y offset in meters                   */
{
/* Place parameters in static storage for common use
  -------------------------------------------------*/
false_easting = false_east;
false_northing = false_north;
R = r;
lon_center = center_long;

return(OK);
}

/* Mollweide inverse equations--mapping x,y to lat,long 
  ----------------------------------------------------*/

winkinv(double x, double y, double *lon, double *lat)
{
int i;
double delta_lon, delta_lat, val, cos_lat, tan_lat, sin_lon_2;
double X, Y;

  *lat = y / 1.414 / R;
  *lon = x / 1.414 / R;

  /* Iterate using the Newton-Raphson method to find *lat
  -----------------------------------------------------*/
  for (i=0;;i++)
     {
     cos_lat = cos(*lat);
     tan_lat = tan(*lat);
     sin_lon_2 = sin(*lon / 2);
     X = 2 * x - 0.636527026 * R * *lon;
     Y = 2 * y - R * *lat;
     val = 2 * Y * sin_lon_2 - X * tan_lat;
     delta_lat = - val / (- 2 * R * sin_lon_2 - X / cos_lat / cos_lat);
     delta_lon = - val / (Y * cos(*lon / 2) + 0.636527026 * R * tan_lat);
     *lat += delta_lat;
     *lon += delta_lon;
     if (fabs(delta_lat) < EPSLN  ||
         fabs(delta_lon) < EPSLN  ) break;
     if (i >= 250) 
       {
       p_error("Iteration failed to converge","Winkel-inverse");
fprintf(stderr, "Iteration failed to converge\n");
       return(241);
       }
     }
  *lat /= 2.0;
  *lon /= 2.0;

return(OK);

}
